const fs = require('fs')

const SVGIcons2SVGFontStream = require('svgicons2svgfont')
const svg2ttf = require('svg2ttf')
const ttf2woff2 = require('ttf2woff2')
const Mustache = require('mustache')

const pack = require('../package.json');
const { downloadFiles } = require('./figma')

const FONT_NAME = 'SampleIcon';

const fontStream = new SVGIcons2SVGFontStream({
  fontName: FONT_NAME,
  fixedWidth: true,
  fontHeight: 24
})

downloadFiles()
  .then((icons) => {
    fontStream
      .pipe(fs.createWriteStream(`./lib/fonts/${FONT_NAME}.svg`))
      .on('finish', () => {
        console.log('Saved SVG successfully!')
        const ttf = svg2ttf(fs.readFileSync(`./lib/fonts/${FONT_NAME}.svg`, 'utf8'), {});
        let content = Buffer.from(ttf.buffer)

        fs.writeFileSync(`./lib/fonts/${FONT_NAME}.ttf`, content)
        console.log('Saved TTF:', content.length, 'bytes')

        let woff2 = ttf2woff2(ttf.buffer);
        content = Buffer.from(woff2.buffer);
        fs.writeFileSync(`./lib/fonts/${FONT_NAME}.wof2`, content)
        console.log('Saved WOFF2:', content.length, 'bytes');

        const props = {
          fontName: FONT_NAME,
          icons: icons.map(icon => ({
            ...icon, unicode: icon.unicode.charCodeAt(0).toString(16)
          })),
          uppercase() {
            return function (text, render) {
              return render(text).toUpperCase();
            }
          }
        };

        fs.promises.readFile('./templates/style.css.mustache', 'utf8')
          .then(async (t) => {
            await fs.promises.writeFile('./css/icon.css', Mustache.render(t, props))
          })
          fs.promises.readFile('./templates/icon.dart.mustache', 'utf8')
          .then(async (t) => {
            await fs.promises.writeFile('./lib/sample_icon.dart', Mustache.render(t, props))
          })
      })
      .on('error', (err) => {
        console.error(err)
        process.exit(1)
      })

    icons.forEach((icon) => {
      const rs = fs.createReadStream(`./src/svg/${icon.filename}`);
      rs.metadata = {
        unicode: [icon.unicode, icon.name],
        name: icon.name
      };
      console.log({
        unicode: [icon.unicode, icon.name],
        name: icon.name
      })

      fontStream.write(rs);
    })
    fontStream.end();
  })
